<?php

namespace Drupal\video_embed_avideo\Plugin\video_embed_field\Provider;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Utility\Error;
use Drupal\video_embed_field\ProviderPluginBase;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provider plugin for AVideo.
 *
 * @VideoEmbedProvider(
 *  id = "avideo",
 *  title = @Translation("AVideo")
 * )
 */
class AVideo extends ProviderPluginBase {

  use LoggerChannelTrait;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Create a plugin with the given input.
   *
   * @param string $configuration
   *   The configuration of the plugin.
   * @param string $plugin_id
   *   The plugin id.
   * @param mixed $plugin_definition
   *   The plugin definition.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   An HTTP client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   *
   * @throws \Exception
   */
  public function __construct($configuration, $plugin_id, $plugin_definition, ClientInterface $http_client, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $http_client);

    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('http_client'),
      $container->get('config.factory')
    );
  }

  /**
   * Get Videohost URL from config.
   */
  public function getVideohost() {
    $config = $this->configFactory->get('video_embed_avideo.settings');
    return $config->get('avideo.hostname');
  }

  /**
   * Get Portalname from config.
   */
  public function getVideohostName() {
    $config = $this->configFactory->get('video_embed_avideo.settings');
    return $config->get('avideo.portalname');
  }

  /**
   * Returns the complete URL.
   */
  public function getEmbedUrl() {
    $config = $this->configFactory->get('video_embed_avideo.settings');
    $iframe = $config->get('avideo.iframe');
    $type = ($iframe == 'full') ? '/v/' : '/videoEmbed/';

    $video_host = $this->getVideohost();
    $video_id = $this->getAvVideoId();

    return $video_host . $type . $video_id;
  }

  /**
   * {@inheritdoc}
   */
  public function getAvVideoId() {
    $videoid = $this->getVideoId();
    $videoid = explode('-', $videoid);
    return end($videoid);
  }

  /**
   * {@inheritdoc}
   */
  public function renderEmbedCode($width, $height, $autoplay) {
    $embed_url = $this->getEmbedUrl();
    if ($autoplay) {
      $embed_url .= '?autoplay=1';
    }
    $portalname = $this->getVideohostName();
    return [
      '#type' => 'video_embed_iframe',
      '#provider' => 'avideo',
      '#url' => $embed_url,
      '#attributes' => [
        'width' => $width,
        'height' => $height,
        'frameborder' => '0',
        'allowfullscreen' => 'allowfullscreen',
        'title' => $this->t('Embedded video from %portal', ['%portal' => $portalname]),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getRemoteThumbnailUrl() {
    $embed_url = $this->getEmbedUrl();

    try {
      $client = $this->httpClient;

      // We need a special user agent to get no default preview image.
      $options = [
        'headers' => [
          'User-Agent' => 'CMS Drupal as Firefox/110',
        ],
      ];
      $response = $client->request('GET', $embed_url, $options);
      $data = (string) $response->getBody();
      if (empty($data)) {
        return FALSE;
      }
      $re = '/<meta property="og:image" content="(?<img>.*)" \/>/m';
      $matches = [];
      preg_match($re, $data, $matches);
      if (!empty($matches['img'])) {
        $url = $matches['img'];
        if (UrlHelper::isValid($url, TRUE)) {
          return $url;
        }
      }
      return FALSE;
    }
    catch (RequestException $e) {
      $logger = $this->getLogger('video_embed_avideo');
      Error::logException($logger, $e);
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getIdFromInput($input) {
    $config = \Drupal::config('video_embed_avideo.settings');
    $video_host = $config->get('avideo.hostname');

    if ((!empty($video_host)) && (strpos($input, $video_host) === 0)) {
      $re = '/.*\/(?<id>\d+)\/?/m';
      $matches = [];
      preg_match($re, $input, $matches);
      if (!empty($matches['id'])) {
        $id = $matches['id'];
        // Hostname is part of the ID to avoid collisions when
        // multiple hosts may be possible in later versions.
        $hash = substr(md5($video_host), 0, 8);
        // Use prefix 'av' to avoid collisions.
        return 'av-' . $hash . '-' . $id;
      }
    }

    return FALSE;
  }

}
